/**

  Simple application to send ping (ICMP Echo request) packets

*/
/* package fi.hut.tcm.jacob.tcpip.test; */

import fi.hut.tcm.jacob.tcpip.icmp.*;
import fi.hut.tcm.jacob.tcpip.ip.*;
import fi.hut.tcm.jacob.tcpip.arp.*;
import fi.hut.tcm.jacob.tcpip.arp.ArpTable;
import fi.hut.tcm.jacob.tcpip.ethernet.*;
import fi.hut.tcm.jacob.tcpip.ethernet.EthernetAdaptor;

import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.*;
import java.io.*;
//import java.util.StringTokenizer;

/**
    SimplePing
    
    
*/
public final class SimplePing
  extends Object {
    
  private final SimpleScheduler scheduler = SimpleScheduler.instance;
  
  private long localEthAddr;
  private int defaultGateway = 0x55443321;

  private static final String itblCfgFilename = new String("itbl.cfg");
  private static final String rtblCfgFilename = new String("rtbl.cfg");

  private IcmpEchoSendAdaptor icmpSendAdap;
  protected ArpTable arpTable;

  public SimplePing(String ifKey) 
  {


    /*************
     * Instances *
     *************/

    // Init interface table and routing table

    
    TcpipInterfaceTable ifTable = null;
    TcpipRoutingTable rTable = null;
    try {
      ifTable = new TcpipInterfaceTable(itblCfgFilename);
      rTable = new TcpipRoutingTable( rtblCfgFilename);
    } catch(IOException e) {
      System.err.println("I/O Error reading config files");
      System.exit(-1);
    } catch(IllegalArgumentException e) {
      System.err.println("Syntax error in config files: " + e);
      System.exit(-1);
    }

    TcpipInterfaceEntry entry = ifTable.get(ifKey);
    if (entry == null) {
      System.err.println("Interface key not found in table");
      System.exit(1);
    }
    int myIpAddr = ((Integer) entry.getIpAddrs().firstElement()).intValue();
    localEthAddr = entry.getEthAddr();
    int mtu = entry.getMtu();

    System.out.println("Starting up with interface " + ifKey + " : ");
    System.out.println("MAC = " + EthernetAddress.toString( localEthAddr) +
		       ", IP = " + IpAddress.toString( myIpAddr ) +
		       " MTU = " + mtu);

    // - * * * * * * * * * * 
    // Ethernet

    // pseudo, machine-internal ethernet:
    /*
    PseudoEthernetAdaptor ethAdapt =
      new PseudoEthernetAdaptor((new EthernetAddress(localEthAddr)).toArray());
    */

    // Real ethernet:

    Vector filteredIps = new Vector();
    filteredIps.addElement(new IpAddress("130.233.45.26")); // clotam
    filteredIps.addElement(new IpAddress("130.233.45.255")); // broadcast
    filteredIps.addElement(new IpAddress("130.233.195.255")); // broadcast

    EthernetAdaptor ethAdapt =
      new EthernetAdaptor( new EthernetAddress(localEthAddr),
			   filteredIps,
			   "/dev/le", 0 );

    /* This is old, bug-workaround code:
    EtherAdaptorAccessor ethAdaptAcc = new EtherAdaptorAccessor();
    Mux ethAdaptMux = new Mux(ethAdaptAcc);
    Terminator ethAdaptTerm = new Terminator();
    */


    // Common to Pseudo- and real ethernet from here on:
    EthernetHeaderSession ethHdrSes = 
      new EthernetHeaderSession(EthernetHeaderState.instance, localEthAddr);
    
    // ARP

    arpTable = new ArpTable();

    EtherAddrAccessor etherAddrAccessor = new EtherAddrAccessor(arpTable);
    Mux etherAddrMux = new Mux(etherAddrAccessor);
    ArpIcmpAdaptor arpIcmpAdapt = new ArpIcmpAdaptor();
    ArpReplyAdaptor arpReplyAdapt = new ArpReplyAdaptor(arpTable, ifTable, ifKey);
    ArpRequestSession arpRequestSes = 
      new ArpRequestSession(arpTable, ArpRequestState.instance, ifTable);
    ArpSelectorAccessor arpSelectorAccessor = new ArpSelectorAccessor();
    Mux arpSelectorMux = new Mux(arpSelectorAccessor);
    Terminator arpSelectorMuxTer = new Terminator(); 


    // IP

    IpChecksumSession ipCSes =
      new IpChecksumSession(IpChecksumState.instance);
    IpFragmentationSession ipFrSes =
      new IpFragmentationSession(IpFragmentationState.instance,
				 ifTable);
    IpRoutingAccessor ipRAcc = new IpRoutingAccessor();
    Mux ipRMux = new Mux(ipRAcc);
    Terminator ipRTer = new Terminator();
    IpRoutingSession ipRSes =
      new IpRoutingSession(IpRoutingOffState.instance,
			   rTable,
			   ifTable);
    IpReassemblySession ipReSes =
      new IpReassemblySession(IpReassemblyState.instance);
    IpProtocolAccessor ipPAcc = new IpProtocolAccessor();
    Mux ipPMux = new Mux(ipPAcc);
    Terminator ipPTer = new Terminator();
    IpHeaderSession ipIcmpHSes =
      new IpHeaderSession(IpHeaderState.instance);



    // ICMP
    icmpSendAdap =
      new IcmpEchoSendAdaptor();
    IcmpEchoAnswerAdaptor icmpAnswAdap =
      new IcmpEchoAnswerAdaptor();
    IcmpSelectorAccessor icmpSeleAcce =
      new IcmpSelectorAccessor();
    Mux icmpSeleMux =
      new Mux(icmpSeleAcce);
    Terminator icmpSeleMuxTerm =
      new Terminator(); 
    IcmpChecksumSession icmpSumSes = 
      new IcmpChecksumSession(IcmpChecksumState.instance);

    // UDP - no UDP

    // Application - no application

    // Stub sessions
    TcpipStubSession icmpStubSes =
      new TcpipStubSession(IpHeaderToNetMessenger.instance,
			   IcmpChecksumFromNetMessenger.instance,
			   null);
    TcpipStubSession ipStubSes =
      //      new TcpipStubSession(null,
      //		   IpChecksumFromNetMessenger.instance,
      //		   IpStubMessenger.instance);
      new TcpipStubSession(null,
			   IpChecksumFromNetMessenger.instance,
			   null);
    
    /*********************
     * Conduit structure *
     *********************/

    try {

      // Pseudo ethernet:  ( + real ethernet now! )

      ethAdapt.attach(Side.A, ethHdrSes);
      ethHdrSes.attach(Side.A, ethAdapt);


      // Real ethernet:  ( no longer needs to be different! )
      /*
      ethArpAdapt.attach(Side.A, ethAdaptMux);
      ethIpAdapt.attach(Side.A, ethAdaptMux);

      ethAdaptMux.attach(Side.A, ethHdrSes);
      ethAdaptMux.attach( new Integer( 0x0800) , ethIpAdapt );
      ethAdaptMux.attach( new Integer( 0x0806) , ethArpAdapt );
      ethAdaptMux.attach(Side.B, ethAdaptTerm);
      ethHdrSes.attach(Side.A, ethAdaptMux);
      */


      // Common to both ethernets:
      ethHdrSes.attach(Side.B, arpSelectorMux);

      // ARP

      arpSelectorMux.attach(Side.A, ethHdrSes);
      arpSelectorMux.attach("1", etherAddrMux);
      arpSelectorMux.attach("2", arpReplyAdapt);
      arpSelectorMux.attach("3", arpRequestSes);
      arpSelectorMux.attach(Side.B, arpSelectorMuxTer);

      arpReplyAdapt.attach(Side.A, arpSelectorMux);
      arpSelectorMuxTer.attach(Side.A, arpSelectorMux);
            
      etherAddrMux.attach(Side.A, ipStubSes);
      etherAddrMux.attach("1", arpSelectorMux);
      etherAddrMux.attach("2", arpRequestSes);
      etherAddrMux.attach(Side.B, arpIcmpAdapt);

      arpRequestSes.attach(Side.A, arpSelectorMux);
      arpRequestSes.attach(Side.B, etherAddrMux);

      arpIcmpAdapt.attach(Side.A, etherAddrMux);

      // ARP-IP Stub

      ipStubSes.attach(Side.A, etherAddrMux);
      ipStubSes.attach(Side.B, ipCSes);
      
      // IP

      ipCSes.attach(Side.A, ipStubSes);
      ipCSes.attach(Side.B, ipFrSes);

      ipFrSes.attach(Side.A, ipCSes);
      ipFrSes.attach(Side.B, ipRMux);

      ipRMux.attach(Side.B, ipRTer);
      ipRMux.attach(ifKey, ipFrSes);
      ipRMux.attach(Side.A, ipRSes);
      
      ipRSes.attach(Side.A, ipRMux);
      ipRSes.attach(Side.B, ipReSes);

      ipReSes.attach(Side.A, ipRSes);
      ipReSes.attach(Side.B, ipPMux);

      ipPMux.attach(Side.A, ipReSes);
      int ICMP_PROTOCOL_ID = 1; // XXX Move this to pepperseed model like udp
      ipPMux.attach(new Byte((byte) ICMP_PROTOCOL_ID), ipIcmpHSes);
      ipPMux.attach(Side.B, ipPTer);

      ipIcmpHSes.attach(Side.A, ipPMux);
      ipIcmpHSes.attach(Side.B, icmpStubSes);

      // IP-ICMP Stub

      icmpStubSes.attach(Side.A, ipIcmpHSes);
      icmpStubSes.attach(Side.B, icmpSumSes);

      // ICMP
      
      icmpSendAdap.attach(Side.A, icmpSeleMux);
      icmpAnswAdap.attach(Side.A, icmpSeleMux);
      
      icmpSeleMux.attach(Side.A, icmpSumSes);
      icmpSeleMux.attach(Side.B, icmpSeleMuxTerm);
      icmpSeleMux.attach(new Integer(8), icmpAnswAdap);
      icmpSeleMux.attach(new Integer(0), icmpSendAdap);

      icmpSumSes.attach(Side.A, icmpStubSes);
      icmpSumSes.attach(Side.B, icmpSeleMux);


    } catch (SideException e) {
      System.err.println("Host " + e + " failed to initialize");
      System.exit(2);
    }
  }

  public void send(int dstIP, int ident, int seq)
  {
    icmpSendAdap.send(dstIP, ident, seq);
  }

  public static void main(String argv[])
  {
    SimplePing me;

    int dstIp;
    int tmp = 0;

    if (argv.length < 1) {
      System.out.println("Usage: java SimplePing iface [dstIP]");
      System.out.println("Example: java SimplePing eth0 1.2.3.4");
      System.exit(2);
    }
    
    me = new SimplePing(argv[0]);

    if (argv.length == 2) {  // otherwise, just answer ping.

      IpAddress ipaddress = new IpAddress(argv[1]);
      dstIp =  ipaddress.toInt();

      try {
	
	for (int i = 1; i < 500; i++ ) {
	  System.out.println("------------> SENDING PING --------------->");
	  me.send( dstIp, 666, i );
	  for (int j = 0; j < 10; j++) {
	    Thread.sleep(100);
	    Thread.currentThread().yield();
	  }
	}
	
	
      } catch (InterruptedException e) {
	// :-P
      }

      me.arpTable.showTable();

    }

  }

}; // SimplePing



