/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  TcpipStubSession (implementation)
    Package: tcpip
    File:    TcpipStubSession.java
    
    


*/
package fi.hut.tcm.jacob.tcpip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.udp.*;
import fi.hut.tcm.jacob.tcpip.ip.*;
import fi.hut.tcm.jacob.tcpip.arp.*;

/**
    TcpipStubSession
    
    A session that glues two Protocols together by providing the initial
    Messenger to a Message that is about to enter the scope of a new
    Protocol.
*/
public class TcpipStubSession
  extends Session {
    protected Messenger sideAMessenger;
    protected Messenger sideBMessenger;
    protected TcpipStubMessenger debugMessenger;

    public TcpipStubSession(Messenger sideAMessenger, Messenger sideBMessenger, TcpipStubMessenger debugMessenger) {
    //+ TcpipStubSession::TcpipStubSession(State, Messenger, Messenger, TcpipStubMessenger)
      super(null);
      this.sideAMessenger = sideAMessenger;
      this.sideBMessenger = sideBMessenger;
      this.debugMessenger = debugMessenger;
    //- TcpipStubSession::TcpipStubSession(State, Messenger, Messenger, TcpipStubMessenger)
    }

    public void apply(Message message) {
    //+ TcpipStubSession::apply(Message)
      if (debugMessenger != null) {
	System.err.println("-----------------------");
	debugMessenger.printHeader(message);
	System.err.println("-----------------------");
      }
      TcpipOutOfBand oob =
	(TcpipOutOfBand) message.getOutOfBand();
      if (oob.isIncoming()) {
	message.setMessenger(this.sideBMessenger);
      } else {
	message.setMessenger(this.sideAMessenger);
      }
    //- TcpipStubSession::apply(Message)
    }

}; //TcpipStubSession
