/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  ArpRequestState (implementation)
    Package: arp
    File:    ArpRequestState.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Hashtable;

/**
    ArpRequestState
    
    Provides methods for sending ARP requests and handling replies: <br>
    - applyData calls APRequestDataMessenger to make
    ARP request to get Ethernet address for packet. <br>
    - applyResponse calls ARPRequestResponseMessenger to
    insert the IP-Ethernet pair in response message to
    the Arp Table.
*/
public class ArpRequestState
  implements State {
    public static ArpRequestState instance = new ArpRequestState();

    /**
        Calls ARPRequestResponseMessenger to
        insert the IP-Ethernet pair in response message to
        the Arp Table
        
    */
    public void applyResponse(Messenger msgr, Message message, Hashtable reqTable, ArpRequestSession my_session) {
    //+ ArpRequestState::applyResponse(Messenger, Message, Hashtable, ArpRequestSession)

      ArpRequestResponseMessenger messenger =
	(ArpRequestResponseMessenger) msgr;
    
      int ipAddress;
    
      // XXX - The en- and dequeuing of messages is not implemented.
      // Now, check the ARP response, and if OK, update our ARP table
      if ( (ipAddress = messenger.handleArpResponse(my_session, message )) != -1) {
	messenger.dequeueMessage( ipAddress, reqTable, my_session );
      }
    
    //- ArpRequestState::applyResponse(Messenger, Message, Hashtable, ArpRequestSession)
    }

    /**
        Calls APRequestDataMessenger to make
        ARP request to get Ethernet address for packet.
        
        @param msgr
        @param message
        @param reqTable
        @param ifaceTable
        @param my_session
        
    */
    public void applyData(Messenger msgr, Message message, Hashtable reqTable, TcpipInterfaceTable ifaceTable, ArpRequestSession my_session) {
    //+ ArpRequestState::applyData(Messenger, Message, Hashtable, TcpipInterfaceTable, ArpRequestSession)

      ArpRequestDataMessenger messenger =
	(ArpRequestDataMessenger) msgr;
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      
      // XXX - The en- and dequeuing of messages is not implemented.
      messenger.enqueueMessage( message, reqTable );

      messenger.makeArpRequest( message, ifaceTable, oob.getNextHopIpAddr(), my_session );

    //- ArpRequestState::applyData(Messenger, Message, Hashtable, TcpipInterfaceTable, ArpRequestSession)
    }

}; //ArpRequestState

