/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  EthernetHeaderToNetMessenger (implementation)
    Package: arp
    File:    EthernetHeaderToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    EthernetHeaderToNetMessenger
    
    Provides the method addEthernetHeader for adding Ethernet header to
    Message.
*/
public class EthernetHeaderToNetMessenger
  implements Messenger {
    public static EthernetHeaderToNetMessenger instance = new EthernetHeaderToNetMessenger();

    /**
        Adds Ethernet header to binarymessage of message by using
        information in out of band part of message.
        @param message
    */
    public void addEthernetHeader(Message message) {
    //+ EthernetHeaderToNetMessenger::addEthernetHeader(Message)

      Object buf = message.getContents(); 
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      long srcEtherAddr;
      long dstEtherAddr;

      srcEtherAddr = oob.getSrcEthAddr();
      dstEtherAddr = oob.getDestEthAddr();
      int frType = oob.getEthFrameType();

      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.print("EthernetHeaderToNetMessenger.addEthernetHeader - ");
        System.out.print("srcEtherAddr = " + EthernetAddress.toString(srcEtherAddr));
	System.out.println(", dstEtherAddr = " + EthernetAddress.toString(dstEtherAddr));
      }

      UtilMsgr.i.prependHeader(buf, Arp.SIZEOFETHERHEADER);

      // The following offsets all assume that we do not see the Ethernet preamble.
      // The ethernet header format is then:
      // || dst addr = 6 octets | src addr = 6 octets | frame type = 2 octets ||
      UtilMsgr.i.set6BytesUnaligned(buf, 0, dstEtherAddr);
      UtilMsgr.i.set6BytesUnaligned(buf, 6, srcEtherAddr);
      UtilMsgr.i.setShort(buf, 6, frType);

    //- EthernetHeaderToNetMessenger::addEthernetHeader(Message)
    }

    /**
        @param message
        @param state
        @param session
        
    */
    public void apply(Message message, State state, Session session) {
    //+ EthernetHeaderToNetMessenger::apply(Message, State, Session)

      EthernetHeaderState ethernetHeaderState = (EthernetHeaderState) state;
      
      TcpipOutOfBand oob =
	(TcpipOutOfBand) message.getOob();
      oob.setSrcEthAddr( ((EthernetHeaderSession) session).etherAddr);

      ethernetHeaderState.applyToNet(this, message);

    //- EthernetHeaderToNetMessenger::apply(Message, State, Session)
    }

}; //EthernetHeaderToNetMessenger

