/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpChecksumState (implementation)
    Package: icmp
    File:    IcmpChecksumState.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpChecksumState
    
    <p>Provides the entry points for incoming and outgoing messages. The
    entry points (methods applyFromNet and applyToNet) calls the
    appropriate Messenger methods for handling the ICMP checksum. Either
    calculating and adding the checksum if the ICMP message is an outgoing
    one, or validating the ICMP checksum if the ICMP message is an incoming
    one.
*/
public class IcmpChecksumState
  implements State {
    /**
         A <code>final</code> reference to the state. There can only be one
        instance of each <code>State</code>.
    */
    public static IcmpChecksumState instance = new IcmpChecksumState();

    /**
        An entry point for messages coming from the net. Validates the ICMP
        packet checksum using methods provided by the
        <code>Messenger</code>.
        
        @param message which <code>Message</code> to apply
        @param messenger the <code>Messenger</code> of this
        <code>Message</code>
        @param message    the message to apply operations on
        @param messenger  which messenger
    */
    public void applyFromNet(Message message, Messenger messenger) {
    //+ IcmpChecksumState::applyFromNet(Message, Messenger)
      IcmpChecksumFromNetMessenger msgr =
	(IcmpChecksumFromNetMessenger) messenger;
      if (msgr.validChecksum(message)) {
	if (Icmp.DebugLevel > 1) {  // #Debug#
	  System.out.println("Checksum OK.");
	}
	return;
      } else {
	// corrupted datagrams are discarded
	// XXX .. but the code is missing...
	System.out.println(" --- Checksum failed! ---");
      }
    //- IcmpChecksumState::applyFromNet(Message, Messenger)
    }

    /**
        An entry point for messages coming from the net. Validates the ICMP
        packet checksum using methods provided by the
        <code>Messenger</code>.
        
        @param message    the message to apply operations on
        @param messenger  which messenger
    */
    public void applyToNet(Message message, Messenger messenger) {
    //+ IcmpChecksumState::applyToNet(Message, Messenger)
      IcmpChecksumToNetMessenger msgr =
	(IcmpChecksumToNetMessenger) messenger;
      msgr.addChecksum(message);
    //- IcmpChecksumState::applyToNet(Message, Messenger)
    }

}; //IcmpChecksumState

