/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpChecksumToNetMessenger (implementation)
    Package: icmp
    File:    IcmpChecksumToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpChecksumToNetMessenger
    
    <p>This is the messenger that contains the logic for ICMP messages
    going out to the network. Contains a method for calculating the
    checksum of this ICMP message and adds it to the message.
*/
public class IcmpChecksumToNetMessenger
  extends IcmpChecksumMessenger
  implements Messenger {
    /**
        There should exist only one <code>Messenger</code> of this kind.
    */
    public static IcmpChecksumToNetMessenger instance = new IcmpChecksumToNetMessenger();

    /**
        Method for applying a message to a state of a specific session.
                
        @param message  <code>Message</code>
        @param state  <code>State</code>
        @param session <code>Session</code>
    */
    public void apply(Message message, State state, Session session) {
    //+ IcmpChecksumToNetMessenger::apply(Message, State, Session)
      IcmpChecksumState stat = (IcmpChecksumState) state;
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();
      oob.setIncoming(false);
      stat.applyToNet(message, this);
    //- IcmpChecksumToNetMessenger::apply(Message, State, Session)
    }

    /**
        Calculates and adds the ICMP checksum to a message.
                
        @param message <code>Message</code>
    */
    public void addChecksum(Message message) {
    //+ IcmpChecksumToNetMessenger::addChecksum(Message)
      setChecksum(message, calculateChecksum(message));
    //- IcmpChecksumToNetMessenger::addChecksum(Message)
    }

}; //IcmpChecksumToNetMessenger

