/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpEchoSendAdaptor (implementation)
    Package: icmp
    File:    IcmpEchoSendAdaptor.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.SimpleScheduler;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpEchoSendAdaptor
    
    <p>Creates ICMP ECHO REQUEST messages and receives echo replies.
*/
public class IcmpEchoSendAdaptor
  extends Adaptor {

    /**
        Receives replies to echo requests.
        
        @param message The <code>Message</code> carrying the echo reply
    */
    public void adapt(Message message) {
    //+ IcmpEchoSendAdaptor::adapt(Message)

      // Incoming replies to packets we have sent should arrive here.

      BinaryBuffer buf = message.getBuffer();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();
      int len = buf.getLength();

      int src = oob.getSrcIpAddr();
      int dst = oob.getDestIpAddr();
      int ident = buf.getShort(4);
      int seq = buf.getShort(6);
      
      System.out.println("<-- Echo Reply: src = " + IpAddress.toString(src) +
			 ", dst = " + IpAddress.toString(dst));
      System.out.println("    ident = " + ident + ", seq = " + seq);
      if (Icmp.DebugLevel > 1) {  // #Debug#
	System.out.println("    Packet dump:");
	buf.hexDump();
      }
      
    //- IcmpEchoSendAdaptor::adapt(Message)
    }

    /**
        Send an ICMP ECHO REQUEST message.
        
        @param    dstIP  destination IP
        @param    ident  packet identfier
        @param    seq  packet sequence number
    */
    public void send(int dstIp, int ident, int seq) {
    //+ IcmpEchoSendAdaptor::send(int, int, int)

    // Maximum size for IP header is 60 octets. (RFC 791).
    // Allow for that, plus some extra header room for ethernet
    // and unforseen requirements. = total of 96 octets.
    // ICMP header for Echo messages is 8 octets + optional data (we use 24)
      // Variables: newBinaryMessage(MAXSIZE, INITIALOFFSET, INITIALLENGTH);
      BinaryBuffer buf =
	BinaryBuffer.newBinaryBuffer(256, 96, 32);
      /* XXX Exercising possible bug here: */
      //      for (int jj = 0; jj < 10; jj++) {
      //	BinaryBuffer dummybuffer =
      //	  BinaryBuffer.newBinaryBuffer(jj*80, jj*40, jj*20);
      //      }
      /* XXXX end debugging code */

      buf.setByte(Icmp.TYPE_OFFSET, 8);
      buf.setByte(Icmp.CODE_OFFSET, 0);

      // data[2] and data[3] are the checksum fields

      buf.setShort(4, ident);
      buf.setShort(6, seq);
      buf.setInt(8, 0x01020304);
      buf.setInt(12, 0x05060708);
      buf.setInt(16, 0x090a0b0c);
      buf.setInt(20, 0x0d0e0f10);
      buf.setInt(24, 0x11121314);
      buf.setInt(28, 0x15161718);
      // buf.setByte(32, 0x19);
      
      TcpipOutOfBand oob = new TcpipOutOfBand();
      oob.setDestIpAddr(dstIp);

      oob.setSrcIpAddr(0);  // Let the IP stack figure out our address.

      Message msg =
	new Message(IcmpChecksumToNetMessenger.instance, oob, buf);
      if (sideA != null) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sideA, msg, this);
	SimpleScheduler.instance.makeRunnable(mt);
      }
    //- IcmpEchoSendAdaptor::send(int, int, int)
    }

}; //IcmpEchoSendAdaptor

