/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpChecksumToNetMessenger (implementation)
    Package: ip
    File:    IpChecksumToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpChecksumToNetMessenger
    
    
*/
public class IpChecksumToNetMessenger
  extends IpMessenger {
    public static IpChecksumToNetMessenger instance = new IpChecksumToNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ IpChecksumToNetMessenger::apply(Message, State, Session)
      IpChecksumState st = (IpChecksumState) state;
      st.applyToNet(message, this);
    //- IpChecksumToNetMessenger::apply(Message, State, Session)
    }

    /**
        Calculates the IP header checksum and adds it to the packet.
    */
    public int calculateChecksum(Message message) {
    //+ IpChecksumToNetMessenger::calculateChecksum(Message)
      BinaryBuffer buffer = message.getBuffer();
      byte hlen = (byte) getHLen(message);
      setChecksum(message, 0);

      int csum = 0;
      for (int i = hlen * 4 - 2; i >= 0; i -= 2) {
	csum += ((int) buffer.getShort(i)) & 0xffff;
      }
      /* 32-bit to 16-bit folding and 1's complement */
      while ((csum >> 16) > 0) {
	csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
      }
      csum = ~csum & 0xffff;
      
      setChecksum(message, csum);
      return csum;
    //- IpChecksumToNetMessenger::calculateChecksum(Message)
    }

}; //IpChecksumToNetMessenger
