/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpFragmentationState (implementation)
    Package: ip
    File:    IpFragmentationState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Enumeration;
import java.util.Vector;

/**
    IpFragmentationState
    
    Provides the methods:
     -applyFromNet()
    Simply forwards the IP packet to the IpRoutingMux.
     -applyToNet(). 
    A state responsible for fragmenting outgoing IP packets that are bigger
    than the MTU. Ignores incoming packets, except for upodating their
    messenger reference.
*/
public class IpFragmentationState
  implements State {
    public static IpFragmentationState instance = new IpFragmentationState();;

    /**
        Simply forwards the IP packet to the IpRoutingMux.
    */
    public void applyFromNet(Message message, Session session, Messenger messenger) {
    //+ IpFragmentationState::applyFromNet(Message, Session, Messenger)
      message.setMessenger(IpRoutingFromNetMessenger.instance);
    //- IpFragmentationState::applyFromNet(Message, Session, Messenger)
    }

    /**
        Calls fragmentationNeeded().
        If fragmentationNeeded() returns Error, the packet is forwarded to
        the IcmpAdaptor.
        Fragments the packet with fragment() if necessary
    */
    public void applyToNet(Message message, Session session, Messenger messenger) {
    //+ IpFragmentationState::applyToNet(Message, Session, Messenger)

      IpFragmentationToNetMessenger msgr =
	(IpFragmentationToNetMessenger) messenger;
      IpFragmentationSession sess = (IpFragmentationSession) session;
      Conduit nextConduit = message.getTransporter().getNext();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      // Use the default IP address of the interface as the source address
      String interfaceId = msgr.getInterfaceId(message);
      TcpipInterfaceEntry entry =
	sess.interfaceTable.get(interfaceId);
      int srcIpAddr = entry.getDefIpAddr();
      msgr.setSrcIpAddr(message, srcIpAddr);
      oob.setSrcIpAddr(srcIpAddr);
      int mtu = entry.getMtu();

      Enumeration fragments = msgr.fragment(message, mtu).elements();
      while (fragments.hasMoreElements()) {
	Message fragment = (Message) fragments.nextElement();
	MessageTransporter trans = 
	  MessageTransporter.newMessageTransporter(nextConduit, 
						   fragment, 
						   sess);
	SimpleScheduler.instance.makeRunnable(trans);
      }
      message.setMessenger(IpChecksumToNetMessenger.instance);
    //- IpFragmentationState::applyToNet(Message, Session, Messenger)
    }

}; //IpFragmentationState

