/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpFragmentationToNetMessenger (implementation)
    Package: ip
    File:    IpFragmentationToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Enumeration;
import java.util.Vector;

/**
    IpFragmentationToNetMessenger
    
    Calls the state's applyToNet() method.
    
*/
public class IpFragmentationToNetMessenger
  extends IpMessenger {
    public static IpFragmentationToNetMessenger instance = new IpFragmentationToNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ IpFragmentationToNetMessenger::apply(Message, State, Session)
      IpFragmentationState st = (IpFragmentationState) state;
      st.applyToNet(message, session, this);
    //- IpFragmentationToNetMessenger::apply(Message, State, Session)
    }

    /**
        Does the hard work of fragmenting those IP packets that are longer
        than the MTU.
        
    */
    public Vector fragment(Message message, int mtu) {
    //+ IpFragmentationToNetMessenger::fragment(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      Vector fragments = new Vector();
      int tlen = getTotalLength(message);
      
      if (tlen > mtu) {
	/* 
	 * Fragmentation needed 
	 */
	if ((getFlags(message) & 0x02) != 0) {
	  /* Don't fragment bit set */
	  // XXX generate ICMP message
	} else {
	  /* Fragment datagram */
          int hlen = getHLen(message) * 4;
	  mtu -= (mtu - hlen) % 8; // Frag. offsets are multiples of 8 octets
	  System.err.println("Dividing " + tlen + " bytes to " + mtu + 
			     " byte fragments");
	  byte[] bufdata = buffer.getData();
	  int offs = hlen;

	  while (offs < tlen) {
	    
	    BinaryBuffer newBuffer = BinaryBuffer.newBinaryBuffer(mtu, 0, mtu);
	    byte[] clonedata = newBuffer.getData();
	    System.arraycopy(bufdata, 0, clonedata, 0, hlen);
	    int datalen = (mtu - hlen < tlen - offs)? mtu - hlen : tlen - offs;
	    System.arraycopy(bufdata, offs, clonedata, hlen, datalen);

	    Message newMessage = 
	      new Message(IpChecksumToNetMessenger.instance, 
			  message.getOutOfBand(),
			  newBuffer);
	    
	    // Fill in fragment specific data
	    setTotalLength(newMessage, datalen + hlen);
	    setFragOffs(newMessage, (offs - hlen) / 8);
	    offs += datalen;
	    if (offs < tlen) {
	      setFlags(newMessage, getFlags(message) | 0x01);
	    }
	    fragments.addElement(newMessage);
	  }
	}
	// The fragmented datagram is discarded
	message.stopTransportation();
      }

      return fragments;
    //- IpFragmentationToNetMessenger::fragment(Message, int)
    }

    public String getInterfaceId(Message message) {
    //+ IpFragmentationToNetMessenger::getInterfaceId(Message)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();
      return oob.getInterfaceId();
    //- IpFragmentationToNetMessenger::getInterfaceId(Message)
    }

}; //IpFragmentationToNetMessenger
