/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpMessenger (implementation)
    Package: ip
    File:    IpMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpMessenger
    
    The base class of all Messengers for the IP protocol. Implements the
    methods for manipulating IP headers.
*/
public abstract class IpMessenger
  implements Messenger {

    public int getVers(Message message) {
    //+ IpMessenger::getVers(Message)
      BinaryBuffer buffer = message.getBuffer();
      return (buffer.getByte(0) >> 4) & 0x0f;
    //- IpMessenger::getVers(Message)
    }

    public void setVers(Message message, int vers) {
    //+ IpMessenger::setVers(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      int bufdata = buffer.getByte(0);
      buffer.setByte(0, ((vers & 0x0f) << 4) | (bufdata & 0x0f));
    //- IpMessenger::setVers(Message, int)
    }

    public int getHLen(Message message) {
    //+ IpMessenger::getHLen(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getByte(0) & 0x0f;
    //- IpMessenger::getHLen(Message)
    }

    public void setHLen(Message message, int hlen) {
    //+ IpMessenger::setHLen(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      int bufdata = buffer.getByte(0);
      buffer.setByte(0, (hlen & 0x0f) | (bufdata & 0xf0));
    //- IpMessenger::setHLen(Message, int)
    }

    public int getServiceType(Message message) {
    //+ IpMessenger::getServiceType(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getByte(1);
    //- IpMessenger::getServiceType(Message)
    }

    public void setServiceType(Message message, int stype) {
    //+ IpMessenger::setServiceType(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setByte(1, stype);
    //- IpMessenger::setServiceType(Message, int)
    }

    public int getTotalLength(Message message) {
    //+ IpMessenger::getTotalLength(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getShort(2);
    //- IpMessenger::getTotalLength(Message)
    }

    public void setTotalLength(Message message, int len) {
    //+ IpMessenger::setTotalLength(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setShort(2, len);
    //- IpMessenger::setTotalLength(Message, int)
    }

    public int getIdentification(Message message) {
    //+ IpMessenger::getIdentification(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getShort(4);
    //- IpMessenger::getIdentification(Message)
    }

    public void setIdentification(Message message, int id) {
    //+ IpMessenger::setIdentification(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setShort(4, id);
    //- IpMessenger::setIdentification(Message, int)
    }

    public int getFlags(Message message) {
    //+ IpMessenger::getFlags(Message)
      BinaryBuffer buffer = message.getBuffer();
      return (buffer.getByte(6) >> 5) & 0x07;
    //- IpMessenger::getFlags(Message)
    }

    public void setFlags(Message message, int flags) {
    //+ IpMessenger::setFlags(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      int bufdata = ((int) buffer.getByte(6)) & 0x1f;
      buffer.setByte(6, ((flags << 5) | bufdata));
    //- IpMessenger::setFlags(Message, int)
    }

    public int getFragOffs(Message message) {
    //+ IpMessenger::getFragOffs(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getShort(6) & 0x1fff;
    //- IpMessenger::getFragOffs(Message)
    }

    public void setFragOffs(Message message, int offs) {
    //+ IpMessenger::setFragOffs(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      int bufdata = ((int) buffer.getShort(6)) & 0xe000;
      buffer.setShort(6, (bufdata | offs));
    //- IpMessenger::setFragOffs(Message, int)
    }

    public int getTTL(Message message) {
    //+ IpMessenger::getTTL(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getByte(8);
    //- IpMessenger::getTTL(Message)
    }

    public void setTTL(Message message, int ttl) {
    //+ IpMessenger::setTTL(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setByte(8, ttl);
    //- IpMessenger::setTTL(Message, int)
    }

    public int getProtocolId(Message message) {
    //+ IpMessenger::getProtocolId(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getByte(9);
    //- IpMessenger::getProtocolId(Message)
    }

    public void setProtocolId(Message message, int prot) {
    //+ IpMessenger::setProtocolId(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setByte(9, prot);
    //- IpMessenger::setProtocolId(Message, int)
    }

    public int getChecksum(Message message) {
    //+ IpMessenger::getChecksum(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getShort(10);
    //- IpMessenger::getChecksum(Message)
    }

    public void setChecksum(Message message, int csum) {
    //+ IpMessenger::setChecksum(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setShort(10, csum);
    //- IpMessenger::setChecksum(Message, int)
    }

    public int getSrcIpAddr(Message message) {
    //+ IpMessenger::getSrcIpAddr(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getInt(12);
    //- IpMessenger::getSrcIpAddr(Message)
    }

    public void setSrcIpAddr(Message message, int ipAddr) {
    //+ IpMessenger::setSrcIpAddr(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setInt(12, ipAddr);
    //- IpMessenger::setSrcIpAddr(Message, int)
    }

    public int getDestIpAddr(Message message) {
    //+ IpMessenger::getDestIpAddr(Message)
      BinaryBuffer buffer = message.getBuffer();
      return buffer.getInt(16);
    //- IpMessenger::getDestIpAddr(Message)
    }

    public void setDestIpAddr(Message message, int ipAddr) {
    //+ IpMessenger::setDestIpAddr(Message, int)
      BinaryBuffer buffer = message.getBuffer();
      buffer.setInt(16, ipAddr);
    //- IpMessenger::setDestIpAddr(Message, int)
    }

}; //IpMessenger
