/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpReassemblyFromNetMessenger (implementation)
    Package: ip
    File:    IpReassemblyFromNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpReassemblyFromNetMessenger
    
    Provides the method reassemble(), that performs the IP re-assembly.
*/
public class IpReassemblyFromNetMessenger
  extends IpMessenger {
    public static IpReassemblyFromNetMessenger instance = new IpReassemblyFromNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ IpReassemblyFromNetMessenger::apply(Message, State, Session)
      IpReassemblyState st = (IpReassemblyState) state;
      st.applyFromNet(message, session, this);
    //- IpReassemblyFromNetMessenger::apply(Message, State, Session)
    }

    public Message reassemble(Message message, Session session) {
    //+ IpReassemblyFromNetMessenger::reassemble(Message, Session)
      Message newMessage = null;

      /* check whether this datagram is a fragment */
      int fragoffs = getFragOffs(message); 
      boolean more_fragments = ((getFlags(message) & 0x01) != 0);

      if (more_fragments || fragoffs > 0) {                  
	/* 
	 * The time of this message (fragment) is due;
	 * invalid fragments are silently discarded and the rest are stored
	 * to reassembly table for later reassembly
	 */
	message.stopTransportation();

	/*
	 * send fragment to reassembly table; if this was the last fragment,
	 * receive the reassembled message and return it to the caller 
	 */
	long id = ((long) getIdentification(message) & 0xffffL) << 32;
	long srcIpAddr = getSrcIpAddr(message) & 0xffffffffL;
	long key = id | srcIpAddr;
	int len = getTotalLength(message) - getHLen(message) * 4;
	IpFragment frag = new IpFragment(message, 
					 fragoffs * 8, 
					 len, 
					 !more_fragments);
	IpReassemblySession sess = (IpReassemblySession) session;
	try {
	  newMessage = sess.reassemblyTable.reassemble(key, frag, this);
	  if (newMessage != null) {
	    System.err.println("Packet reassembled");
	  }
	  
	} catch (IpInvalidFragmentOffsetException e) {}
      }
      return newMessage;
    //- IpReassemblyFromNetMessenger::reassemble(Message, Session)
    }

    protected void append(Message message, Message to) {
    //+ IpReassemblyFromNetMessenger::append(Message, Message)
      BinaryBuffer srcBuf = message.getBuffer();
      BinaryBuffer dstBuf = to.getBuffer();
      int tlen = getTotalLength(message);
      int hlen = getHLen(message) * 4;
      int doffs = dstBuf.getLength();
      dstBuf.appendTrailer(tlen - hlen);
      BinaryBuffer.copyBuffer(srcBuf, hlen, dstBuf, doffs, tlen - hlen);
    //- IpReassemblyFromNetMessenger::append(Message, Message)
    }

}; //IpReassemblyFromNetMessenger

