/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpReassemblyTable (implementation)
    Package: ip
    File:    IpReassemblyTable.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import java.util.Hashtable;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpReassemblyTable
    
    
*/
public class IpReassemblyTable {
    protected Hashtable entries;

    public IpReassemblyTable() {
    //+ IpReassemblyTable::IpReassemblyTable( )
      entries = new Hashtable();
    //- IpReassemblyTable::IpReassemblyTable( )
    }

    public Message reassemble(long key, IpFragment newFrag, Messenger messenger)
    throws IpInvalidFragmentOffsetException {
    //+ IpReassemblyTable::reassemble(long, IpFragment, Messenger)

      IpReassemblyEntry entry =
	(IpReassemblyEntry) entries.get(new Long(key));

      /*
       * first fragment of a new datagram?
       */
      if (entry == null) {
	entry = new IpReassemblyEntry(); 
	entries.put(new Long(key), entry);
      }

      /* 
       * 'complete' will remain 'true' iff there are no "holes" in
       * the combined data of the fragment list and the last fragment
       * in the list has "more fragments" bit set to 0)
       */
      boolean complete = true;

      IpFragment frag, lfrag = null;
      for (frag = entry.fragments; frag.next != null; frag = frag.next) {
	if (newFrag != null && newFrag.offs < frag.next.offs) {
	  /* disallow overlapping fragments */
	  if (newFrag.offs + newFrag.len > frag.next.offs || 
	      newFrag.offs < frag.offs + frag.len) {
	    System.err.println("Invalid fragment offset. ");
	    System.err.println("Prev  offs: " + frag.offs + " len: " + frag.len);
	    System.err.println("This  offs: " + newFrag.offs + " len: " + newFrag.len);
	    System.err.println("Next  offs: " + frag.next.offs + " len: " + frag.next.len);
	    throw new IpInvalidFragmentOffsetException("XXX");
	  }
	  
	  /* insert new fragment and make it the current one in the loop */
	  newFrag.next = frag.next;
	  frag.next = newFrag;
	  newFrag = null;
	  if (!complete) {
	    return null;
	  }
	}
        if (frag.offs + frag.len < frag.next.offs && frag.next.offs < 65536) {
	  complete = false; // A "hole" in data
	}
	lfrag = frag;
      }
      
      if (!lfrag.last || !complete ) {
	return null;   // have to wait for more fragments
      }
      
      /* 
       * the datagram is complete; reassemble and pass it to the caller
       */
      System.err.println("Reassembling datagram");
      
      IpReassemblyFromNetMessenger msgr =
	(IpReassemblyFromNetMessenger) messenger;
      int hlen = msgr.getHLen(lfrag.msg) * 4;
      int tlen = lfrag.offs + lfrag.len + hlen;
      TcpipOutOfBand oob = (TcpipOutOfBand) lfrag.msg.getOutOfBand();
      BinaryBuffer buffer = BinaryBuffer.newBinaryBuffer(tlen, 0, hlen);
      Message newMessage =
	new Message(IpHeaderFromNetMessenger.instance, oob, buffer);
      BinaryBuffer.copyBuffer(lfrag.msg.getBuffer(), 0, 
			      buffer, 0, 
			      hlen);
      
      frag = entry.fragments;
      do {
	frag = frag.next;
	System.err.println("Frag: " + frag.offs + "-" + 
			   (frag.offs + frag.len));
	msgr.append(frag.msg, newMessage);
      } while (frag != lfrag);

      return newMessage;
    //- IpReassemblyTable::reassemble(long, IpFragment, Messenger)
    }

}; //IpReassemblyTable

