/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingState (implementation)
    Package: ip
    File:    IpRoutingState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpRoutingState
    
    This is the superclass of RoutingOnState and RoutingOffState, providing
    their common applyToNet() implementation and applyFromNet() interface.
*/
public abstract class IpRoutingState
  implements State {


  public abstract void applyFromNet(Message message, Session session,
				    Messenger messenger);

    /**
      Calls the Messenger's destinationLocal() method which tells if the
      destination IP address belongs to this machine. If the destination
      is local, then the message is sent back to the ReassemblySession.
      Otherwise, the messenger's setNextHopIp() method is called, and the
      packet is sent on  its way to the IpRoutingMux.
      
      */
  public void applyToNet(Message message, Session session, 
			 Messenger messenger) {
  //+ IpRoutingState::applyToNet(Message, Session, Messenger)
    IpRoutingToNetMessenger  msgr = (IpRoutingToNetMessenger) messenger;
    int dst = msgr.getDestIpAddr(message);
    IpRoutingSession sess = (IpRoutingSession) session;

    /*
     * Resolve the "next-hop" IP address, based on the ultimate destination IP
     */
    TcpipRoutingEntry entry = sess.routingTable.get(dst);
    if (entry == null) {
      System.err.println("Couldn't resolve nexthop address. " +
			 "Packet discarded.");
      message.stopTransportation();
      return;
    }
    // nexthop is the default router if nothing more appropriate was found
    // and zero if the destination is directly reachable
    int nexthop = entry.getNexthop();
    if (nexthop == 0) {
      nexthop = dst;
    }
    msgr.setNextHopIpAddr(message, nexthop);
    msgr.setInterfaceId(message, entry.getInterfaceId());

    message.setMessenger(IpFragmentationToNetMessenger.instance);
  //- IpRoutingState::applyToNet(Message, Session, Messenger)
  }
    
}; //IpRoutingState

