
#define VERSION "0.11"

/*
 * fifoman.c
 *
 * This is the companion program to the PseudoEthernetAdaptor
 * module in the Java Conduits TCP/IP project. The PseudoEthernet
 * Adaptors "talk" to this program through FIFO:s in the file
 * system. A script to quickly make these fifos, makefifos.sh, should
 * be found in the same place as this file.
 *
 * Copyright(C) 1998 Mikael Suokas & Employer
 */

/*
 * This program reads the controlfifo and distributes ethernet packets.
 * in effect, this program is "the network". Clients register themselves
 * by writing MAGIC_START + '2' + own_ethernet_address + MAGIC_END.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

char *BASE_ETH_NAME = "aa:bb:cc:dd:ee:0";

int main(int argc, char *argv[])
{
  int in_fd[10];
  int out_fd[10];
  int registered_flag[10];
  FILE *CONTROL;
  int i, opcode, dst_addr, src_addr;
  int size_msb, size_lsb, size;
  char fnam[32];
  struct stat statbuf;
  char inbuf[2048];
  char addrbuf[2];

  printf("fifoman version %s\n", VERSION);
  strcpy(fnam, BASE_ETH_NAME);
  fnam[strlen(BASE_ETH_NAME)+ 1] = '\0';
  for(i = 0; i < 10; i++) {
    fnam[strlen(BASE_ETH_NAME)] = '0' + i;
    if ((in_fd[i] = open(fnam, O_RDONLY | O_NONBLOCK)) == -1) {
      fprintf(stderr, "%s: Error opening file/FIFO %s for reading: %s\n",
	      argv[0], fnam, strerror(errno));
      exit(1);
    }
    if (fstat(in_fd[i], &statbuf) != 0) {
      fprintf(stderr, "%s: Unable to stat file/FIFO %s: %s\n",
	      argv[0], fnam, strerror(errno));
      exit(2);
    }
    if ( ! S_ISFIFO(statbuf.st_mode) ) {
      fprintf(stderr, "%s: %s is not a FIFO!\n", argv[0], fnam);
      exit(3);
    }
    if ((out_fd[i] = open(fnam, O_WRONLY)) == -1) {
      fprintf(stderr, "%s: Error opening FIFO %s for writing: %s\n",
	      argv[0], fnam, strerror(errno));
      exit(4);
    }
  }
  if ((CONTROL = fopen("controlfifo", "r" )) == NULL) {
    fprintf(stderr, "%s: Error opening file/FIFO %s for reading: %s\n",
	    argv[0], "controlfifo", strerror(errno));
    exit(5);
  }

  printf("Type CTRL-C to exit...\n");
  fflush(stdout);
  for(;;) {
    if ((i = fgetc(CONTROL)) == EOF) {
      fprintf(stderr, "%s: Error reading control FIFO: %s\n", argv[0], strerror(errno));
      exit(6);
    }
    if (i != 89) {
      fprintf(stderr, "%s: Corrupt data on control FIFO (MAGIC_START)! ABORTING!:\n", argv[0]);
      fprintf(stderr, "Got %d instead of 89\n", i);
      exit(7);
    }
    if ((opcode = fgetc(CONTROL)) == EOF) {
      fprintf(stderr, "%s: Error reading control FIFO: %s\n", argv[0], strerror(errno));
      exit(6);
    }
    /* For now, just read away the first 5 bytes, relying on the last byte for the address */
    for (i = 0; i < 5; i++) {
      fgetc(CONTROL);
    }
    if ((dst_addr = fgetc(CONTROL)) == EOF) {
      fprintf(stderr, "%s: Error reading control FIFO: %s\n", argv[0], strerror(errno));
      exit(6);
    }
    switch (opcode) {
    case 1 :   /* normal send */
      size_msb = fgetc(CONTROL);
      size_lsb = fgetc(CONTROL);
      size = size_msb * 256 + size_lsb;
      if (fread(&inbuf, 1, size, CONTROL) != size ) {
	 fprintf(stderr, "%s: Error reading control FIFO! ABORTING!:\n", argv[0]);
	 exit(9);
      }
      printf("R"); fflush(stdout);
      src_addr = inbuf[11];
      if (dst_addr == 255) {                   /* Broadcast?? */
	printf(" B[%d]", src_addr);
	for( i = 0; i < 10; i++) {
	  if ((registered_flag[i] == 1) && (i != src_addr)) {
	    addrbuf[0] = size_msb;
	    addrbuf[1] = size_lsb;
	    write(out_fd[i], addrbuf, 2);
	    if (write(out_fd[i], inbuf, size) == -1) {
	      fprintf(stderr, "%s: Error writing to address %d\n", argv[0], i);
	      exit(1);
	    }
	    printf("S"); fflush(stdout);
	  }
	}
	printf(" "); fflush(stdout);
      } else {                            /* non-broadcast */
	if ( registered_flag[dst_addr] == 1) {
	  addrbuf[0] = size_msb;
	  addrbuf[1] = size_lsb;
	  write(out_fd[dst_addr], addrbuf, 2);
	  if (write(out_fd[dst_addr], inbuf, size) == -1) {
	    fprintf(stderr, "%s: Error writing to address %d\n", argv[0], dst_addr);
	    exit(1);
	  }
	  printf("S"); fflush(stdout);
	}
      }
      break;
    case 2 :   /* register */
      registered_flag[dst_addr] = 1;
      printf("\nHost registered at address %d\n", dst_addr);
      break;
    case 3 :   /* unregister */
      registered_flag[dst_addr] = 0;
      printf("\nHost unregistered at address %d\n", dst_addr);
      break;
    default:
      fprintf(stderr, "%s: Corrupt data on control FIFO (invalid opcode)! ABORTING!\n", argv[0]);
      exit(8);
      break;
    }
    if (( i = fgetc(CONTROL)) == EOF) {
      fprintf(stderr, "%s: Error reading control FIFO! ABORTING!:\n", argv[0]);
      exit(8);
    }
    if (i != 213) {   /* finally, check the MAGIC_END byte */
      fprintf(stderr, "%s: Corrupt data on control FIFO (MAGIC_END)! ABORTING!\n", argv[0]);
      exit(1);
    }
    /* Just loop around, waiting for CTRL-C */
    /* we're too lazy to close any files; let the system handle that */
  }
  exit(0);
}

